#!/usr/bin/env bash
#
# Licensed Materials - Property of IBM
#
# IBM Financial Crimes Insight with Watson PID 5900-A0H
# IBM Financial Crimes Insight with Watson, Private PID 5737-E41
#
# (C) Copyright IBM Corp. 2017  All rights reserved.
#
# The source code for this program is not published or otherwise divested
# of its trade secrets, irrespective of what has been deposited with the
# U. S. Copyright Office.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#

#----------------------------------------------------------------------------------
# Intended maintenance state machine:
#
# Starting from NORMAL operation:
# [User requests maintenance] set-maintenance-mode.sh NORMAL
# NORMAL -> REQUEST_MAINT
#
# The readiness probe detects the maintenance request:
# REQUEST_MAINT -> QUIESCING -> QUIESCED
#
# [Maintence complete] set-maintenance-mode.sh MAINT_COMPLETE
# QUIESCED -> MAINT_COMPLETE
#
# The readiness probe detects the maintenance complete notification
# MAINT_COMPLETE -> RESUMING -> NORMAL
#
# The readiness probe detects that a required service is not available
# NORMAL -> SERVICE_DOWN -> QUIESCING_SVC -> QUIESCED_SVC
#
# The readiness probe detects that all required services are now available:
# QUIESCED_SVC -> SERVICE_UP -> RESUMING -> NORMAL

MAINTENANCE_STATES=(NORMAL REQUEST_MAINT QUIESCING QUIESCED MAINT_COMPLETE RESUMING QUIESCING_SVC QUIESCED_SVC SERVICE_DOWN SERVICE_UP)

for ((i=0; i < ${#MAINTENANCE_STATES[@]}; i++)); do
	name=${MAINTENANCE_STATES[i]}
	declare -r ${name}=${i}
done

maintenance_code_to_string() {
	local -i maintenance_code=${1}; shift
	local -i rc=127
	local maintenance_string=""

	if [[ $maintenance_code -ge 0 && $maintenance_code -lt ${#MAINTENANCE_STATES[@]} ]]; then
		maintenance_string="${MAINTENANCE_STATES[$maintenance_code]}"
		rc=0
	fi

	echo "$maintenance_string"

	return $rc
}

maintenance_string_to_code() {
	local maintenance_string="${1}"; shift
        local count=${#MAINTENANCE_STATES[@]}
        local -i rc=127

        for ((i=0; i < count; i++ )); do
                [ "${MAINTENANCE_STATES[$i]}" == "${maintenance_string}" ] && rc=$i && break
        done

        return $rc
}
