#!/usr/bin/env bash

SCRIPT_DIR=$(cd -P -- "$(dirname -- "$0")" && pwd -P)

. "${SCRIPT_DIR}/container-script-location.sh"
. "${SCRIPT_DIR}/maintenance-states.sh"
. "${SCRIPT_DIR}/service-has-active-nodes.sh"

main() {
	(bash "$SCRIPT_DIR/get-maintenance-mode.sh") > /dev/null
	local -i maintenance_mode=$?

	if [[ ${maintenance_mode} -eq ${NORMAL} || ${maintenance_mode} -eq ${QUIESCED_SVC} ]]; then
		local -i any_service_down=0
		while [[ $# -gt 0 ]]; do
			service-has-active-nodes "$1"
			if [[ $? -ne 0 ]]; then
				any_service_down=1
				break
			fi
			shift
		done

		if [[ ${maintenance_mode} -eq ${NORMAL} && ${any_service_down} -eq 1 ]]; then
			("$SCRIPT_DIR/set-maintenance-mode.sh" SERVICE_DOWN)
		elif [[ ${maintenance_mode} -eq ${QUIESCED_SVC} && ${any_service_down} -eq 0 ]]; then
			("$SCRIPT_DIR/set-maintenance-mode.sh" SERVICE_UP)
		fi
	fi

	(bash "$SCRIPT_DIR/get-maintenance-mode.sh") > /dev/null
	maintenance_mode=$?

	case $maintenance_mode in
	$NORMAL)
		("${CONTAINER_SCRIPTS}/healthcheck.sh")
		return $?
		;;
	$REQUEST_MAINT)
		("$SCRIPT_DIR/set-maintenance-mode.sh" "QUIESCING")
		"${CONTAINER_SCRIPTS}/stop.sh"
		("$SCRIPT_DIR/set-maintenance-mode.sh" "QUIESCED")
		return 1
		;;
	$SERVICE_DOWN)
		("$SCRIPT_DIR/set-maintenance-mode.sh" "QUIESCING_SVC")
		"${CONTAINER_SCRIPTS}/stop.sh"
		("$SCRIPT_DIR/set-maintenance-mode.sh" "QUIESCED_SVC")
		return 1
		;;
	$MAINT_COMPLETE)
		;&
	$SERVICE_UP)
		("$SCRIPT_DIR/set-maintenance-mode.sh" "RESUMING")
		"${CONTAINER_SCRIPTS}/start.sh" -i
		("$SCRIPT_DIR/set-maintenance-mode.sh" "NORMAL")
		("${CONTAINER_SCRIPTS}/healthcheck.sh")
		return $?
		;;
	*)
		return 1
		;;
	esac
}


main "$@"

