#!/usr/bin/env bash

wget_cmd=$(command -v wget)
wget_missing=$?
curl_cmd=$(command -v curl)
curl_missing=$?

authorization_header="Authorization: Bearer $(cat /var/run/secrets/kubernetes.io/serviceaccount/token)"

service-has-active-nodes() {
	local -r service_descriptor="${1}"; shift
	local -r kube_namespace=${service_descriptor%%/*}
	local -r kube_service=${service_descriptor##*/}
	local -r url="https://${KUBERNETES_SERVICE_HOST}/api/v1/namespaces/${kube_namespace}/endpoints/${kube_service}"

	local -i active_node_count

	if [[ $curl_missing -eq 0 ]]; then
		active_node_count=$( "$curl_cmd" -s \
			--cacert /var/run/secrets/kubernetes.io/serviceaccount/ca.crt \
			--header "$authorization_header" \
			"${url}" | awk '/"addresses":/{flag=1;next}/\]/{flag=0}flag' | grep -c \"ip\")
	elif [[ $wget_missing -eq 0 ]]; then
		active_node_count=$( "$wget_cmd" -q -O - \
			--ca-certificate=/var/run/secrets/kubernetes.io/serviceaccount/ca.crt \
			--header="$authorization_header" \
			"${url}" | awk '/"addresses":/{flag=1;next}/\]/{flag=0}flag' | grep -c \"ip\")
	else
		echo "Can't find a command to make REST call"
		return 1
	fi

	[[ $active_node_count -gt 0 ]] && return 0 || return 1
}

