#!/usr/bin/env bash
# 
# Licensed Materials - Property of IBM
#
# IBM Financial Crimes Insight with Watson PID 5900-A0H  
# IBM Financial Crimes Insight with Watson, Private PID 5737-E41
# 
# (C) Copyright IBM Corp. 2017  All rights reserved.
# 
# The source code for this program is not published or otherwise divested  
# of its trade secrets, irrespective of what has been deposited with the  
# U. S. Copyright Office. 
# 
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#

#----------------------------------------------------------------------------------
# Set flags for container maintenance requests.  eg: set-maintenance-mode.sh QUIESCED"
#
# See maintenance-states.sh for details about the states.
#
# Return code:
#     0 - success
#   126 - invalid state transition
#   127 - invalid argument
# other - error creating maintenance mode PID file  
#----------------------------------------------------------------------------------
SCRIPT_DIR=$(cd -P -- "$(dirname -- "$0")" && pwd -P)
. "${SCRIPT_DIR}/maintenance-states.sh"

maint_PID_file="/tmp/MAINTENANCE_MODE"

action=${1}
action=${action^^}

state_transition_validation() {
	local -i new_state="$1"; shift
	bash "$SCRIPT_DIR/get-maintenance-mode.sh" > /dev/null
	local -i old_state=$?
	local rc=1

	if [[ "$new_state" -eq "$old_state" ]] || \
	   [[ "$old_state" -eq $NORMAL         && "$new_state" -eq $REQUEST_MAINT ]] || \
	   [[ "$old_state" -eq $NORMAL         && "$new_state" -eq $SERVICE_DOWN ]] || \
	   [[ "$old_state" -eq $REQUEST_MAINT  && "$new_state" -eq $QUIESCING ]] || \
	   [[ "$old_state" -eq $QUIESCING      && "$new_state" -eq $QUIESCED ]] || \
	   [[ "$old_state" -eq $QUIESCED       && "$new_state" -eq $MAINT_COMPLETE ]] || \
	   [[ "$old_state" -eq $MAINT_COMPLETE && "$new_state" -eq $RESUMING ]] || \
	   [[ "$old_state" -eq $RESUMING       && "$new_state" -eq $NORMAL ]] || \
	   [[ "$old_state" -eq $SERVICE_DOWN   && "$new_state" -eq $QUIESCING_SVC ]] || \
	   [[ "$old_state" -eq $QUIESCING_SVC  && "$new_state" -eq $QUIESCED_SVC ]] || \
	   [[ "$old_state" -eq $QUIESCED_SVC   && "$new_state" -eq $SERVICE_UP ]] || \
	   [[ "$old_state" -eq $SERVICE_UP     && "$new_state" -eq $RESUMING ]] || \
	   [[ 0 -eq 1 ]]
	then
		rc=0
	fi

	return $rc
}

maintenance_string_to_code "${action}"
rc=$? 

if [[ $rc -ne 127 ]]; then
	state_transition_validation $rc
	if [[ $? -eq 0 ]]; then
		if [[ $rc -eq 0 ]] && [[ ! -f "${maint_PID_file}" ]]; then
			: # Don't bother creating the file (implicitly 'normal' operation)
		else
			echo "$rc" > "${maint_PID_file}"
			echo "OK"
			rc=$?
		fi
	else
		echo "Cannot switch to $action"
		rc=126
	fi
else
	if [[ $action != "" ]]; then
		echo "$action is not recognized"
	fi
	echo "Valid modes:"
	for mode in "${MAINTENANCE_STATES[@]}"; do
		echo "$mode"
	done
fi

exit $rc


